@@ -1,7 +1,8 @@
 /*
-    $Id: bincue.c,v 1.15 2005/02/17 07:03:37 rocky Exp $
+    $Id: bincue.c,v 1.19 2006/02/13 11:00:53 rocky Exp $
 
-    Copyright (C) 2002, 2003, 2004, 2005 Rocky Bernstein <rocky@panix.com>
+    Copyright (C) 2002, 2003, 2004, 2005, 2006 
+    Rocky Bernstein <rocky@panix.com>
     Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
     cue parsing routine adapted from cuetools
     Copyright (C) 2003 Svend Sanjay Sorensen <ssorensen@fastmail.fm>
@@ -26,7 +27,7 @@
    (*.cue).
 */
 
-static const char _rcsid[] = "$Id: bincue.c,v 1.15 2005/02/17 07:03:37 rocky Exp $";
+static const char _rcsid[] = "$Id: bincue.c,v 1.19 2006/02/13 11:00:53 rocky Exp $";
 
 #include "image.h"
 #include "cdio_assert.h"
@@ -944,7 +945,7 @@ cdio_get_devices_bincue (void)
 char *
 cdio_get_default_device_bincue(void)
 {
-  char **drives = cdio_get_devices_nrg();
+  char **drives = cdio_get_devices_bincue();
   char *drive = (drives[0] == NULL) ? NULL : strdup(drives[0]);
   cdio_free_device_list(drives);
   return drive;
@@ -953,9 +954,15 @@ cdio_get_default_device_bincue(void)
 static bool
 get_hwinfo_bincue ( const CdIo_t *p_cdio, /*out*/ cdio_hwinfo_t *hw_info)
 {
-  strcpy(hw_info->psz_vendor, "libcdio");
-  strcpy(hw_info->psz_model, "CDRWIN");
-  strcpy(hw_info->psz_revision, CDIO_VERSION);
+  strncpy(hw_info->psz_vendor, "libcdio",
+	  sizeof(hw_info->psz_vendor)-1);
+  hw_info->psz_vendor[sizeof(hw_info->psz_vendor)-1] = '\0';
+  strncpy(hw_info->psz_model, "CDRWIN",
+	 sizeof(hw_info->psz_model)-1);
+  hw_info->psz_model[sizeof(hw_info->psz_model)-1] = '\0';
+  strncpy(hw_info->psz_revision, CDIO_VERSION, 
+	  sizeof(hw_info->psz_revision)-1);
+  hw_info->psz_revision[sizeof(hw_info->psz_revision)-1] = '\0';
   return true;
   
 }
@@ -1166,6 +1173,8 @@ cdio_open_cue (const char *psz_cue_name)
   _funcs.read_mode2_sectors    = _read_mode2_sectors_bincue;
   _funcs.run_mmc_cmd           =  NULL;
   _funcs.set_arg               = _set_arg_image;
+  _funcs.set_speed             = cdio_generic_unimplemented_set_speed;
+  _funcs.set_blocksize         = cdio_generic_unimplemented_set_blocksize;
   
   if (NULL == psz_cue_name) return NULL;
   
