@@ -1,8 +1,8 @@
 /*  Common Multimedia Command (MMC) routines.
 
-    $Id: mmc.c,v 1.30 2006/02/13 11:00:53 rocky Exp $
+    $Id: mmc.c,v 1.37 2008/03/04 10:27:54 rocky Exp $
 
-    Copyright (C) 2004, 2005, 2006 Rocky Bernstein <rocky@panix.com>
+    Copyright (C) 2004, 2005, 2006, 2007, 2008 Rocky Bernstein <rocky@gnu.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -56,6 +56,9 @@ cdio_mmc_feature_profile_t   debug_cdio_mmc_feature_profile;
 cdio_mmc_get_conf_t          debug_cdio_mmc_get_conf;
 cdio_mmc_gpcmd_t             debug_cdio_mmc_gpcmd;
 cdio_mmc_read_sub_state_t    debug_cdio_mmc_read_sub_state;
+cdio_mmc_read_cd_type_t      debug_cdio_mmc_read_cd_type;
+cdio_mmc_readtoc_t           debug_cdio_mmc_readtoc;
+cdio_mmc_mode_page_t         debug_cdio_mmc_mode_page;
 
 /*************************************************************************
   MMC CdIo Operations which a driver may use. 
@@ -166,6 +169,13 @@ get_mcn_mmc (const void *p_user_data)
   return mmc_get_mcn( p_env->cdio );
 }
 
+driver_return_code_t
+get_tray_status (const void *p_user_data)
+{
+  const generic_img_private_t *p_env = p_user_data;
+  return mmc_get_tray_status( p_env->cdio );
+}
+
 /*! Read sectors using SCSI-MMC GPCMD_READ_CD.
    Can read only up to 25 blocks.
 */
@@ -188,7 +198,8 @@ set_blocksize_mmc (void *p_user_data, uint16_t i_blocksize)
   return mmc_set_blocksize(p_env->cdio, i_blocksize);
 }
 
-/* Set CD-ROM drive speed (via MMC) */
+/* Set the drive speed Set the drive speed in K bytes per second. (via
+   MMC). */
 driver_return_code_t
 set_speed_mmc (void *p_user_data, int i_speed)
 {
@@ -197,6 +208,15 @@ set_speed_mmc (void *p_user_data, int i_speed)
   return mmc_set_speed( p_env->cdio, i_speed );
 }
 
+/* Set the drive speed in CD-ROM speed units (via MMC). */
+driver_return_code_t
+set_drive_speed_mmc (void *p_user_data, int i_Kbs_speed)
+{
+  generic_img_private_t *p_env = p_user_data;
+  if (!p_env) return DRIVER_OP_UNINIT;
+  return mmc_set_drive_speed( p_env->cdio, i_Kbs_speed );
+}
+
 /** Get the output port volumes and port selections used on AUDIO PLAY
     commands via a MMC MODE SENSE command using the CD Audio Control
     Page.
@@ -829,12 +849,12 @@ mmc_get_hwinfo ( const CdIo_t *p_cdio,
 }
 
 /*! 
-  Find out if media has changed since the last call.
+  Return results of media status
   @param p_cdio the CD object to be acted upon.
-  @return 1 if media has changed since last call, 0 if not. Error
+  @return DRIVER_OP_SUCCESS (0) if we got the status.
   return codes are the same as driver_return_code_t
    */
-int mmc_get_media_changed(const CdIo_t *p_cdio)
+int mmc_get_event_status(const CdIo_t *p_cdio, uint8_t out_buf[2])
 {
   mmc_cdb_t cdb = {{0, }};
   uint8_t buf[8] = { 0, };
@@ -856,11 +876,30 @@ int mmc_get_media_changed(const CdIo_t *p_cdio)
                                     &cdb, SCSI_MMC_DATA_READ, 
                                     sizeof(buf), buf);
   if(i_status == 0) {
-    return 0 != (buf[4] & 0x02);
+    out_buf[0] = buf[4];
+    out_buf[1] = buf[5];
+    return DRIVER_OP_SUCCESS;
   }
   return DRIVER_OP_ERROR;
 }
 
+/*! 
+  Find out if media has changed since the last call.
+  @param p_cdio the CD object to be acted upon.
+  @return 1 if media has changed since last call, 0 if not. Error
+  return codes are the same as driver_return_code_t
+   */
+int mmc_get_media_changed(const CdIo_t *p_cdio)
+{
+  uint8_t status_buf[2];
+  int i_status;
+
+  i_status = mmc_get_event_status(p_cdio, status_buf);
+  if (i_status != DRIVER_OP_SUCCESS)
+    return i_status;
+  return (status_buf[0] & 0x02) ? 1 : 0;
+}
+
 char *
 mmc_get_mcn ( const CdIo_t *p_cdio )
 {
@@ -868,6 +907,23 @@ mmc_get_mcn ( const CdIo_t *p_cdio )
   return mmc_get_mcn_private (p_cdio->env, p_cdio->op.run_mmc_cmd );
 }
 
+/*! 
+  Find out if media tray is open or closed.
+  @param p_cdio the CD object to be acted upon.
+  @return 1 if media is open, 0 if closed. Error
+  return codes are the same as driver_return_code_t
+   */
+int mmc_get_tray_status(const CdIo_t *p_cdio)
+{
+  uint8_t status_buf[2];
+  int i_status;
+
+  i_status = mmc_get_event_status(p_cdio, status_buf);
+  if (i_status != DRIVER_OP_SUCCESS)
+    return i_status;
+  return (status_buf[1] & 0x01) ? 1 : 0;
+}
+
 /*!
   Run a MMC command. 
  
@@ -894,6 +950,21 @@ mmc_run_cmd( const CdIo_t *p_cdio, unsigned int i_timeout_ms,
 				     p_cdb, e_direction, i_buf, p_buf);
 }
 
+/* Added by SukkoPera to allow CDB length to be specified manually */
+driver_return_code_t
+mmc_run_cmd_len( const CdIo_t *p_cdio, unsigned int i_timeout_ms,
+                  const mmc_cdb_t *p_cdb, unsigned int i_cdb,
+                  cdio_mmc_direction_t e_direction, unsigned int i_buf,
+                  /*in/out*/ void *p_buf )
+{
+  if (!p_cdio) return DRIVER_OP_UNINIT;
+  if (!p_cdio->op.run_mmc_cmd) return DRIVER_OP_UNSUPPORTED;
+  return p_cdio->op.run_mmc_cmd(p_cdio->env, i_timeout_ms,
+                                     i_cdb,
+                                     p_cdb, e_direction, i_buf, p_buf);
+}
+
+
 /*! Return the byte size returned on a MMC READ command (e.g. READ_10,
     READ_MSF, ..)
 */
@@ -968,16 +1039,18 @@ mmc_start_stop_media(const CdIo_t *p_cdio, bool b_eject, bool b_immediate,
                                  SCSI_MMC_DATA_WRITE, 0, &buf);
 }
 
-#if 0
-/*!
+/**
  * Close tray using a MMC START STOP command.
  */
-driver_return_code_t
-mmc_close_tray( const CdIo_t *p_cdio )
+driver_return_code_t 
+mmc_close_tray( CdIo_t *p_cdio )
 {
-  return mmc_start_stop_media(p_cdio, false, false, 0);
+  if (p_cdio) {
+    return mmc_start_stop_media(p_cdio, false, false, 0);
+  } else {
+    return DRIVER_OP_ERROR;
+  }
 }
-#endif
 
 /*!
  Eject using MMC commands. If CD-ROM is "locked" we'll unlock it.
@@ -1289,7 +1362,7 @@ mmc_read_data_sectors ( CdIo_t *p_cdio, void *p_buf,
                      true,  /* return user data */
                      false, /* return EDC ECC */
                      false, /* return C2 Error information */
-                     0,     /* suchannel selection bits */
+                     0,     /* subchannel selection bits */
                      ISO_BLOCKSIZE, /* blocksize*/ 
                      i_blocks       /* Number of blocks. */);
 
@@ -1336,13 +1409,37 @@ mmc_set_blocksize ( const CdIo_t *p_cdio, uint16_t i_blocksize)
 
 
 /*!
-  Set the drive speed. 
+  Set the drive speed in CD-ROM speed units.
+  
+  @param p_cdio	   CD structure set by cdio_open().
+  @param i_drive_speed   speed in CD-ROM speed units. Note this
+                         not Kbytes/sec as would be used in the MMC spec or
+	                 in mmc_set_speed(). To convert CD-ROM speed units 
+		         to Kbs, multiply the number by 176 (for raw data)
+		         and by 150 (for filesystem data). On many CD-ROM 
+		         drives, specifying a value too large will result 
+		         in using the fastest speed.
+
+  @return the drive speed if greater than 0. -1 if we had an error. is -2
+  returned if this is not implemented for the current driver.
+
+   @see cdio_set_speed and mmc_set_speed
+*/
+driver_return_code_t 
+mmc_set_drive_speed( const CdIo_t *p_cdio, int i_drive_speed )
+{
+  return mmc_set_speed(p_cdio, i_drive_speed * 176);
+}
+
+  
+/*!
+  Set the drive speed in K bytes per second. 
   
   @return the drive speed if greater than 0. -1 if we had an error. is -2
   returned if this is not implemented for the current driver.
 */
 int
-mmc_set_speed( const CdIo_t *p_cdio, int i_speed )
+mmc_set_speed( const CdIo_t *p_cdio, int i_Kbs_speed )
 
 {
   uint8_t buf[14] = { 0, };
@@ -1352,18 +1449,18 @@ mmc_set_speed( const CdIo_t *p_cdio, int i_speed )
      will return an error - it's part of the ATAPI specs. Therefore, 
      test and stop early. */
 
-  if ( i_speed < 1 ) return -1;
+  if ( i_Kbs_speed < 176 ) return -1;
   
   memset(&cdb, 0, sizeof(mmc_cdb_t));
   CDIO_MMC_SET_COMMAND(cdb.field, CDIO_MMC_GPCMD_SET_SPEED);
-  CDIO_MMC_SET_LEN16(cdb.field, 2, i_speed);
+  CDIO_MMC_SET_LEN16(cdb.field, 2, i_Kbs_speed);
   /* Some drives like the Creative 24x CDRW require one to set a
      nonzero write speed or else one gets an error back.  Some
      specifications have setting the value 0xfffff indicate setting to
      the maximum allowable speed.
   */
   CDIO_MMC_SET_LEN16(cdb.field, 4, 0xffff);
-  return mmc_run_cmd(p_cdio, 2000, &cdb, SCSI_MMC_DATA_READ, 
+  return mmc_run_cmd(p_cdio, 2000, &cdb, SCSI_MMC_DATA_WRITE, 
                      sizeof(buf), buf);
 }
 
