@@ -1,8 +1,8 @@
 /*  private MMC helper routines.
 
-    $Id: mmc_private.h,v 1.9 2005/10/21 11:13:54 rocky Exp $
+    $Id: mmc_private.h,v 1.11 2006/10/11 12:38:18 rocky Exp $
 
-    Copyright (C) 2004, 2005 Rocky Bernstein <rocky@panix.com>
+    Copyright (C) 2004, 2005, 2006 Rocky Bernstein <rocky@cpan.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -69,6 +69,8 @@ int get_media_changed_mmc (const void *p_user_data);
 
 char *get_mcn_mmc (const void *p_user_data);
 
+driver_return_code_t get_tray_status (const void *p_user_data);
+
 /*! Read just the user data part of some sort of data sector (via 
     mmc_read_cd). 
 
@@ -97,8 +99,11 @@ char *get_mcn_mmc (const void *p_user_data);
 driver_return_code_t set_blocksize_mmc (void *p_user_data, 
 					uint16_t i_blocksize);
 
-/* Set CD-ROM drive speed (via MMC) */
-driver_return_code_t set_speed_mmc (void *p_user_data, int i_speed);
+/* Set the drive speed in CD-ROM speed units (via MMC). */
+driver_return_code_t set_drive_speed_mmc (void *p_user_data, int i_speed);
+
+/* Set CD-ROM drive speed  in K bytes per second. (via MMC) */
+driver_return_code_t set_speed_mmc (void *p_user_data, int i_Kbs_speed);
 
 /***********************************************************
   Miscellaenous other "private" routines. Probably need
