@@ -1,8 +1,8 @@
 /*
-    $Id: iso9660.c,v 1.27 2006/09/26 18:07:59 flameeyes Exp $
+    $Id: iso9660.c,v 1.34 2007/11/16 22:50:21 flameeyes Exp $
 
     Copyright (C) 2000 Herbert Valerio Riedel <hvr@gnu.org>
-    Copyright (C) 2003, 2004, 2005, 2006 Rocky Bernstein <rocky@gnu.org>
+    Copyright (C) 2003, 2004, 2005, 2006, 2007 Rocky Bernstein <rocky@gnu.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -43,6 +43,9 @@ const char ISO_STANDARD_ID[] = {'C', 'D', '0', '0', '1'};
 #ifdef HAVE_STDIO_H
 #include <stdio.h>
 #endif
+#ifdef HAVE_LIMITS_H
+#include <limits.h>
+#endif
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
@@ -54,7 +57,7 @@ const char ISO_STANDARD_ID[] = {'C', 'D', '0', '0', '1'};
 #include <errno.h>
 #endif
 
-static const char _rcsid[] = "$Id: iso9660.c,v 1.27 2006/09/26 18:07:59 flameeyes Exp $";
+static const char _rcsid[] = "$Id: iso9660.c,v 1.34 2007/11/16 22:50:21 flameeyes Exp $";
 
 /* Variables to hold debugger-helping enumerations */
 enum iso_enum1_s     iso_enums1;
@@ -164,14 +167,31 @@ iso9660_get_dtime (const iso9660_dtime_t *idr_date, bool b_localtime,
   return true;
 }
 
+/* 
+   A note regarding the strange strtol() testing below as pointed out SMS.
+   From man strtol:
+
+     If an underflow occurs, strtol() returns LONG_MIN. If an overflow
+     occurs, strtol() returns LONG_MAX.  In both cases, errno is set to
+     ERANGE.
+
+   This implies that one should only look at errno if the value is
+   LONG_MIN or LONG_MAX.
+*/
+
 #define set_ltime_field(TM_FIELD, LT_FIELD, ADD_CONSTANT)               \
   {                                                                     \
-    char num[10];                                                       \
+    char num[10]; long tmp;                                             \
     memcpy(num, p_ldate->LT_FIELD, sizeof(p_ldate->LT_FIELD));          \
-    num[sizeof(p_ldate->LT_FIELD)+1] = '\0';                            \
-    p_tm->TM_FIELD = strtol(num,                                        \
-                            (char **)NULL, 10)+ADD_CONSTANT;            \
-    if (0 != errno) return false;                                       \
+    num[sizeof(p_ldate->LT_FIELD)] = '\0';                              \
+    errno = 0;                                                          \
+    tmp = strtol(num,                                                   \
+                 (char **)NULL, 10);                                    \
+    if ( tmp < INT_MIN || tmp > INT_MAX ||                              \
+         ((unsigned long)tmp + ADD_CONSTANT) > INT_MAX ||               \
+         (tmp + ADD_CONSTANT) < INT_MIN )                               \
+      return false;                                                     \
+    p_tm->TM_FIELD = tmp + ADD_CONSTANT;                                \
   }
     
 /*!
