@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004, 2005, 2006 Rocky Bernstein <rocky@panix.com>
+ * Copyright (C) 2004, 2005, 2006, 2007, 2008 Rocky Bernstein <rocky@gnu.org>
  *           (C) 1998 Monty <xiphmont@mit.edu>
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -20,33 +20,6 @@
  *
  * See ChangeLog for recent changes.
  *
- * last changes:
- *   22.01.98 - first version
- *   15.02.98 - alpha 2: juggled two includes from interface/low_interface.h
- *                       that move contents in Linux 2.1
- *                       Linked status bar to isatty to avoid it appearing
- *                       in a redirected file.
- *                       Played with making TOC less verbose.
- *   04.04.98 - alpha 3: zillions of bugfixes, also added MMC and IDE_SCSI
- *                       emulation support
- *   05.04.98 - alpha 4: Segfault fix, cosmetic repairs
- *   05.04.98 - alpha 5: another segfault fix, cosmetic repairs, 
- *                       Gadi Oxman provided code to identify/fix nonstandard
- *                       ATAPI CDROMs 
- *   07.04.98 - alpha 6: Bugfixes to autodetection
- *   18.06.98 - alpha 7: Additional SCSI error handling code
- *                       cosmetic fixes
- *                       segfault fixes
- *                       new sync/silence code, smaller fft      
- *   15.07.98 - alpha 8: More new SCSI code, better error recovery
- *                       more segfault fixes (two linux bugs, one my fault)
- *                       Fixup reporting fixes, smilie fixes.
- *                       AIFF support (in addition to AIFC)
- *                       Path parsing fixes
- *   Changes are becoming TNTC. Will resume the log at beta.
- *
- *   09.04.04 - conversion to use libcdio. See top-level Changelog for
- *              libcdio history.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -61,6 +34,10 @@
 # include <stdarg.h>
 #endif
 
+#ifdef HAVE_LIMITS_H
+#include <limits.h>
+#endif
+
 #ifdef HAVE_STDLIB_H
 # include <stdlib.h>
 #endif
@@ -313,6 +290,7 @@ long callscript=0;
 
 static int skipped_flag=0;
 static int abort_on_skip=0;
+FILE *logfile = NULL;
 
 #if TRACE_PARANOIA
 static void
@@ -364,10 +342,11 @@ callback(long int inpos, paranoia_cb_mode_t function)
   static int stimeout=0;
   const char *smilie="= :-)";
   
-  if(callscript)
-    fprintf(stderr,"##: %d [%s] @ %ld\n",
-	    function,(function>=-2&&function<=13?callback_strings[function+2]:
-		      ""),inpos);
+  if (callscript)
+    fprintf(stderr, "##: %d [%s] @ %ld\n",
+	    function, ((int) function >= -2 && (int) function <= 13 ?
+		       callback_strings[function+2] : ""),
+	    inpos);
 
   if(!quiet){
     long test;
@@ -552,6 +531,12 @@ callback(long int inpos, paranoia_cb_mode_t function)
 	}
    
 	fprintf(stderr,buffer);
+
+	if (logfile != NULL && function==-1) {
+	  fprintf(logfile,buffer+1);
+	  fprintf(logfile,"\n\n");
+	  fflush(logfile);
+	}
       }
     }
   }
@@ -562,7 +547,7 @@ callback(long int inpos, paranoia_cb_mode_t function)
 }
 #endif /* !TRACE_PARANOIA */
 
-const char *optstring = "aBcCd:efg:hi:m:n:o:O:pqQrRsS:Tt:VvwWx:XYZz::";
+const char *optstring = "aBcCd:efg:hi:l:m:n:o:O:pqQrRsS:Tt:VvwWx:XYZz::";
 
 struct option options [] = {
 	{"abort-on-skip",             no_argument,       NULL, 'X'},
@@ -578,11 +563,11 @@ struct option options [] = {
 	{"force-read-speed",          required_argument, NULL, 'S'},
 	{"force-search-overlap",      required_argument, NULL, 'o'},
 	{"help",                      no_argument,       NULL, 'h'},
+ 	{"log-summary",               required_argument, NULL, 'l'},
 	{"mmc-timeout",               required_argument, NULL, 'm'},
 	{"never-skip",                optional_argument, NULL, 'z'},
 	{"output-aifc",               no_argument,       NULL, 'a'},
 	{"output-aiff",               no_argument,       NULL, 'f'},
-	{"output-info",               required_argument, NULL, 'i'},
 	{"output-raw",                no_argument,       NULL, 'p'},
 	{"output-raw-big-endian",     no_argument,       NULL, 'R'},
 	{"output-raw-little-endian",  no_argument,       NULL, 'r'},
@@ -605,7 +590,6 @@ static cdrom_drive_t    *d        = NULL;
 static cdrom_paranoia_t *p        = NULL;
 static char             *span     = NULL;
 static char *force_cdrom_device   = NULL;
-static char *info_file            = NULL;
 
 #define free_and_null(p) \
   free(p);		 \
@@ -632,7 +616,10 @@ cleanup (void)
   if (d) cdda_close(d);
   free_and_null(force_cdrom_device);
   free_and_null(span);
-  free_and_null(info_file);
+  if(logfile && logfile != stdout) {
+      fclose(logfile);
+      logfile = NULL;
+    }
 }
 
 /* Returns true if we have an integer argument. 
@@ -652,8 +639,9 @@ get_int_arg(char c, long int *pi_arg)
 	    " but not found. Option ignored\n", c);
     return false;
   }
+  errno = 0;
   i_arg = strtol(optarg, &p_end, 10);
-  if (errno == ERANGE) {
+  if ( (LONG_MIN == i_arg || LONG_MAX == i_arg) && (0 != errno) ) {
     fprintf(stderr,
 	    "Value '%s' for option -%c out of range. Value %ld "
 	    "used instead.\n", optarg, c, i_arg);
@@ -737,9 +725,18 @@ main(int argc,char *argv[])
     case 'h':
       usage(stdout);
       exit(0);
-    case 'i':
-      if(info_file)free(info_file);
-      info_file=strdup(info_file);
+    case 'l':
+      if(logfile && logfile != stdout)fclose(logfile);
+      if(!strcmp(optarg,"-"))
+	logfile=stdout;
+      else{
+	logfile=fopen(optarg,"w");
+	if(logfile==NULL){
+	  report3("Cannot open log summary file %s: %s",(char*)optarg,
+		  strerror(errno));
+	  exit(1);
+	}
+      }
       break;
     case 'm':
       {
@@ -833,6 +830,18 @@ main(int argc,char *argv[])
       exit(1);
     }
   }
+  
+  if(logfile){
+    /* log command line and version */
+    int i;
+    for (i = 0; i < argc; i++) 
+      fprintf(logfile,"%s ",argv[i]);
+    fprintf(logfile,"\n");
+    
+    fprintf(logfile,VERSION);
+    fprintf(logfile,"\n");
+    fflush(logfile);
+  }
 
   if(optind>=argc && !query_only){
     if(batch)
@@ -913,6 +922,8 @@ main(int argc,char *argv[])
       report("Search overlap sectors must be 0<= n <=75\n");
       cdda_close(d);
       d=NULL;
+      if(logfile && logfile != stdout)
+        fclose(logfile);
       exit(1);
     }
     {
@@ -1052,7 +1063,7 @@ main(int argc,char *argv[])
       int i;
 
       for( i=track1; i<=track2; i++ )
-	if(!cdda_track_audiop(d,i)){
+	if(i != 0 && !cdda_track_audiop(d,i)){
 	  report("Selected span contains non audio tracks.  Aborting.\n\n");
 	  exit(1);
 	}
@@ -1132,6 +1143,10 @@ main(int argc,char *argv[])
 	      report("Are you sure you wanted 'batch' "
 		     "(-B) output with stdout?");
 	    report("outputting to stdout\n");
+	    if(logfile){
+	      fprintf(logfile,"outputting to stdout\n");
+	      fflush(logfile);
+	    }
 	    outfile_name[0]='\0';
 	  } else {
 	    char path[256];
@@ -1175,6 +1190,10 @@ main(int argc,char *argv[])
 	      exit(1);
 	    }
 	    report2("outputting to %s\n", outfile_name);
+	    if(logfile){
+	      fprintf(logfile,"outputting to %s\n",outfile_name);
+	      fflush(logfile);
+	    }
 	  }
 	} else {
 	  /* default */
@@ -1205,6 +1224,11 @@ main(int argc,char *argv[])
 	    exit(1);
 	  }
 	  report2("outputting to %s\n", outfile_name);
+	  if(logfile){
+	    fprintf(logfile,"outputting to %s\n",outfile_name);
+	    fflush(logfile);
+	  }
+
 	}
 	
 	switch(output_type) {
