@@ -1,5 +1,5 @@
 #!/bin/sh
-#$Id: check_cue.sh.in,v 1.29 2005/09/15 06:36:01 rocky Exp $
+#$Id: check_cue.sh.in,v 1.31 2007/12/28 02:11:01 rocky Exp $
 # Tests to see that BIN/CUE and cdrdao TOC file iamge reading is correct
 # (via cd-info).
 
@@ -15,7 +15,11 @@ if test "X$top_srcdir" = "X" ; then
   top_srcdir=`pwd`/..
 fi
 
-. ${srcdir}/check_common_fn
+if test "X$top_builddir" = "X" ; then
+  top_builddir=`pwd`/..
+fi
+
+. ${top_builddir}/test/check_common_fn
 
 if test ! -x $top_srcdir/src/cd-info ; then
   exit 77
