@@ -1,7 +1,7 @@
 /*
-    $Id: testiso9660.c,v 1.17 2006/07/30 14:10:16 rocky Exp $
+    $Id: testiso9660.c,v 1.20 2007/11/16 22:44:57 flameeyes Exp $
 
-    Copyright (C) 2003, 2006 Rocky Bernstein <rocky@panix.com>
+    Copyright (C) 2003, 2006, 2007  Rocky Bernstein <rocky@gnu.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -253,11 +253,15 @@ main (int argc, const char *argv[])
     }
     p_tm = gmtime(&now);
     iso9660_set_dtime(p_tm, &dtime);
-    iso9660_get_dtime(&dtime, false, &tm);
+    if (!iso9660_get_dtime(&dtime, false, &tm)) {
+      printf("Error returned by iso9660_get_dtime\n");
+      return 42;
+    }
+    
     if ( memcmp(p_tm, &tm, sizeof(tm)) != 0 ) {
       printf("GMT time retrieved with iso9660_get_dtime() not same as that\n");
       printf("set with iso9660_set_dtime().\n");
-      return 42;
+      return 43;
     }
     
     {
@@ -265,13 +269,20 @@ main (int argc, const char *argv[])
 	p_tm = localtime(&now);
 	t1 = mktime(p_tm);
 	iso9660_set_ltime(p_tm, &ltime);
-	iso9660_get_ltime(&ltime, &tm);
+	  
+	if (!iso9660_get_ltime(&ltime, &tm)) {
+	  printf("Problem running iso9660_get_ltime\n");
+	  return 44;
+	}
+
+#ifdef FIXED	
 	t2 = mktime(&tm);
 	if ( t1 != t2  && ! time_compare(p_tm, &tm) ) {
 	  printf("local time retrieved with iso9660_get_ltime() not\n");
 	  printf("same as that set with iso9660_set_ltime().\n");
-	  return 43;
+	  return 45;
 	}
+#endif
 
 	p_tm = gmtime(&now);
 	t1 = mktime(p_tm);
@@ -281,7 +292,7 @@ main (int argc, const char *argv[])
 	if ( t1 != t2 && ! time_compare(p_tm, &tm) ) {
 	  printf("GMT time retrieved with iso9660_get_ltime() not\n");
 	  printf("same as that set with iso9660_set_ltime().\n");
-	  return 44;
+	  return 46;
 	}
     }
   }
